package ch.rakudave.jnetmap.view.preferences;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.TitledBorder;

import ch.rakudave.jnetmap.util.IO;
import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.Settings;
import ch.rakudave.jnetmap.util.SwingHelper;

/**
 * @author rakudave
 */
@SuppressWarnings("serial")
public class GeneralPanel extends PreferencePanel {
	@SuppressWarnings("rawtypes")
	private JComboBox languageSelector;
	private SpinnerNumberModel undoSize, updateThreads;
	private JCheckBox restoreMaps, restordeWH, restordeXY;
	

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public GeneralPanel(final Window parent) {
		title = Lang.getNoHTML("preferences.general");
		// Language
			JPanel languageWrapper = new JPanel(new GridLayout(1, 2, 5, 5)) {{
				setMaximumSize(new Dimension(500,25));
				setBorder(BorderFactory.createTitledBorder(Lang.getNoHTML("preferences.general.language") + 
						" " + Lang.getNoHTML("message.restart")));
				languageSelector = new JComboBox(Lang.getLanguages());
					languageSelector.setSelectedItem(Lang.currentLanguage());
				add(languageSelector);
				add(SwingHelper.createAlignedButton(new AbstractAction(Lang.get("action.install"), Icons.get("install")) {
					@Override
					public void actionPerformed(ActionEvent e) {
						File f = SwingHelper.openDialog(parent, null);
						if (f == null) return;
						IO.copy(f, new File(IO.langDir, f.getName()), null);
						languageSelector.setModel(new DefaultComboBoxModel(Lang.getLanguages()));
							languageSelector.setSelectedItem(Lang.currentLanguage());
							languageSelector.showPopup();
					}
				}));
			}};
		// Restore
			JPanel restorePanel = new JPanel(new GridLayout(0, 1, 5, 5));
				restorePanel.setMaximumSize(new Dimension(2000, 60));
				restorePanel.setBorder(new TitledBorder(Lang.getNoHTML("preferences.appearance.window")));
				restordeWH = new JCheckBox(Lang.get("preferences.appearance.remember.wh"), Settings.getBoolean("mapview.remember.wh", true));
				restordeXY = new JCheckBox(Lang.get("preferences.appearance.remember.xy"), Settings.getBoolean("mapview.remember.xy", true));
				restorePanel.add(restordeWH);
				restorePanel.add(restordeXY);
		// Undo history size
			JPanel undoWrapper = new JPanel(new GridLayout(1, 2, 5, 5)) {{
				setMaximumSize(new Dimension(500,25));
				add(new JLabel(Lang.get("preferences.general.undosize")));
				undoSize = new SpinnerNumberModel(Settings.getInt("commandhistory.size", 20), 5, 100, 1);
				add(new JSpinner(undoSize));
			}};
		// Update threads
			JPanel updateThreadWrapper = new JPanel(new GridLayout(1, 2, 5, 5)) {{
				setMaximumSize(new Dimension(500,25));
				add(new JLabel(Lang.get("preferences.general.updatethreads")));
				updateThreads = new SpinnerNumberModel(Settings.getInt("status.update.threads", 5), 1, 20, 1);
				add(new JSpinner(updateThreads));
			}};
		// Restore maps
			JPanel restoreWrapper = new JPanel(new FlowLayout(FlowLayout.LEADING));
				restoreMaps = new JCheckBox(Lang.get("maps.restore"), Settings.getBoolean("maps.restore", false));
				restoreWrapper.add(restoreMaps);
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		add(languageWrapper);
		add(Box.createVerticalStrut(5));
		add(restorePanel);
		add(Box.createVerticalStrut(5));
		add(undoWrapper);
		add(Box.createVerticalStrut(5));
		add(updateThreadWrapper);
		add(Box.createVerticalStrut(5));
		add(restoreWrapper);
		add(Box.createVerticalGlue());
	}

	@Override
	public String getParentTitle() {
		return (parentTitle == null)?"":parentTitle;
	}		

	@Override
	public String getTitle() {
		return (title == null)?"":title;
	}

	@Override
	public void save() {
		Settings.put("lang", languageSelector.getSelectedItem().toString());
		Settings.put("mapview.remember.wh", restordeWH.isSelected());
		Settings.put("mapview.remember.xy", restordeXY.isSelected());
		Settings.put("status.update.threads", updateThreads.getNumber().intValue());
		Settings.put("commandhistory.size", undoSize.getNumber().intValue());
		Settings.put("maps.restore", restoreMaps.isSelected());
	}
}
